//
//  matrices.c
//  matricesStructuresAllocationDynamique
//
//  Created by soldano on 14/09/2014.
//  Copyright (c) 2014 soldano. All rights reserved.
//

#include "matrices.h"

matrice_t construitMatrice(int nl, int nc)
{
    /* construit une matrice de dimensions nl nc */
    int i;
    matrice_t m;
    
    m.nl=nl;
    m.nc=nc;
    m.t=(int **) malloc(m.nl*sizeof(int*));//alloue un tableau de nl tableaux
    
    for (i=0;i<m.nl;i++){
        m.t[i]= (int *) malloc (m.nc* sizeof (int));
    }
    return m;
}

matrice_t dupliqueMatrice(matrice_t m)
{
    /* duplique la matrice m */
    /* m2=dupliqueMatrice(m1) permet de copier m1 dans m2 */
    /* sans que les modifications (ensuite) de m2 ne modifient également m1 */
    int i,j;
    matrice_t md;
    //
    md=construitMatrice(m.nl, m.nc);
    for (i=0; i<md.nl; i++)
        for (j=0; j<md.nc; j++)
            md.t[i][j]=m.t[i][j];
    return md;
}


void detruitMatrice(matrice_t m)
{
    /* desalloue la mémoire de la matrice m */
    int i;
    //
    for (i=0; i<m.nl; i++)
        free(m.t[i]);
    free(m.t);
    m.nl=0;
    m.nc=0;
    return ;
}

matrice_t lireMatrice(int nl, int nc)
{
    /* Alloue, lit et renvoie une matrice de dimensions nl nc */
    int i,j;
    matrice_t m;
    
    m=construitMatrice(nl,nc);
    
    for (i=0;i<m.nl;i++){
        for (j=0;j<m.nc;j++){
            scanf("%d",&m.t[i][j]);
        }
    }
    //    printf("fin\n");
    return m;
}

void afficheMatrice(matrice_t m)
{
    /* affiche la matrice m */
    int i,j;
    for (i=0;i<m.nl;i++){
        for (j=0;j<m.nc;j++){
            printf("%d ",m.t[i][j]);
        }
        printf("\n");
    }
    return;
    
}


matrice_t produitMatrices(matrice_t m1, matrice_t m2)
{
    /* renvoie le produit de deux matrices m1 et m2 */
    int i,j,k;
    matrice_t mr;
    mr=construitMatrice(m1.nl,m2.nc);
    for (i=0;i<mr.nl;i++){
        for (j=0;j<mr.nc;j++){
            mr.t[i][j]=0;
            for(k=0;k<m1.nc;k=k+1){
                mr.t[i][j]= mr.t[i][j]+ m1.t[i][k]*m2.t[k][j];
            }
        }
    }
    return mr;
}

matrice_t sommeMatrices(matrice_t m1, matrice_t m2)
{
    /* renvoie la somme de deux matrices m1 et m2 */
    int i,j;
    matrice_t mr;
    mr=construitMatrice(m1.nl,m1.nc);
    
    for (i=0;i<mr.nl;i++){
        for (j=0;j<mr.nc;j++){
            mr.t[i][j]= m1.t[i][j]+ m2.t[i][j];
        }
    }
    return mr;
}

matrice_t transpose(matrice_t m)
{
    /*	renvoie la transposee d'une matrice m	*/
    int i,j;
    matrice_t mr;
    mr=construitMatrice(m.nc,m.nl);
    for (i=0;i<mr.nl;i++){
        for (j=0;j<mr.nc;j++){
            mr.t[i][j]=m.t[j][i];
        }
    }
    return mr;
}
